@@ -28,7 +28,6 @@ import shutil
 import subprocess
 import sys
 from collections import defaultdict, namedtuple
-from functools import partial
 
 from reportlab.pdfbase.pdfmetrics import registerFont, stringWidth
 from reportlab.pdfbase.ttfonts import TTFError, TTFont
@@ -36,7 +35,7 @@ from reportlab.pdfgen.canvas import FILL_EVEN_ODD, FILL_NON_ZERO
 from reportlab.pdfgen.pdfimages import PDFImage
 from reportlab.graphics.shapes import (
     _CLOSEPATH, Circle, Drawing, Ellipse, Group, Image, Line, Path, PolyLine,
-    Polygon, Rect, String,
+    Polygon, Rect, SolidShape, String,
 )
 from reportlab.lib import colors
 from reportlab.lib.units import pica, toLength
@@ -50,12 +49,16 @@ from .utils import (
     normalise_svg_path,
 )
 
-__version__ = '0.9.0'
+__version__ = '0.9.1'
 __license__ = 'LGPL 3'
 __author__ = 'Dinu Gherman'
-__date__ = '2018-12-08'
+__date__ = '2019-06-22'
 
 XML_NS = 'http://www.w3.org/XML/1998/namespace'
+
+# A sentinel to identify a situation where a node reference a fragment not yet defined.
+DELAYED = object()
+
 STANDARD_FONT_NAMES = (
     'Times-Roman', 'Times-Italic', 'Times-Bold', 'Times-BoldItalic',
     'Helvetica', 'Helvetica-Oblique', 'Helvetica-Bold', 'Helvetica-BoldOblique',
@@ -69,7 +72,7 @@ logger = logging.getLogger(__name__)
 
 Box = namedtuple('Box', ['x', 'y', 'width', 'height'])
 
-split_whitespace = re.compile('[^ \t\r\n\f]+').findall
+split_whitespace = re.compile(r'[^ \t\r\n\f]+').findall
 
 
 def find_font(font_name):
@@ -172,6 +175,10 @@ class AttributeConverter(object):
 
     def __init__(self):
         self.css_rules = None
+        self.main_box = None
+
+    def set_box(self, main_box):
+        self.main_box = main_box
 
     def parseMultiAttributes(self, line):
         """Try parsing compound attribute string.
@@ -273,7 +280,7 @@ class AttributeConverter(object):
         ops = ops.replace(',', ' ').split()
 
         if len(ops) != len(indices):
-            logger.warn("Unable to parse transform expression '%s'" % svgAttr)
+            logger.warning("Unable to parse transform expression '%s'" % svgAttr)
             return []
 
         result = []
@@ -298,7 +305,7 @@ class Svg2RlgAttributeConverter(AttributeConverter):
     def split_attr_list(attr):
         return shlex.split(attr.strip().replace(',', ' '))
 
-    def convertLength(self, svgAttr, percentOf=100, em_base=12):
+    def convertLength(self, svgAttr, em_base=12, attr_name=None):
         "Convert length to points."
 
         text = svgAttr
@@ -309,8 +316,20 @@ class Svg2RlgAttributeConverter(AttributeConverter):
             text = text.replace(',', ' ').split()[0]
 
         if text.endswith('%'):
-            logger.debug("Fiddling length unit: %")
-            return float(text[:-1]) / 100 * percentOf
+            if self.main_box is None:
+                logger.error("Unable to resolve percentage unit without a main box")
+                return float(text[:-1])
+            if attr_name is None:
+                logger.error("Unable to resolve percentage unit without knowing the node name")
+                return float(text[:-1])
+            if attr_name in ('x', 'cx', 'x1', 'x2', 'width'):
+                full = self.main_box.width
+            elif attr_name in ('y', 'cy', 'y1', 'y2', 'height'):
+                full = self.main_box.height
+            else:
+                logger.error("Unable to detect if node '%s' is width or height" % attr_name)
+                return float(text[:-1])
+            return float(text[:-1]) / 100 * full
         elif text.endswith("pc"):
             return float(text[:-2]) * pica
         elif text.endswith("pt"):
@@ -321,7 +340,7 @@ class Svg2RlgAttributeConverter(AttributeConverter):
             return float(text[:-2])
 
         if "ex" in text:
-            logger.warn("Ignoring unit ex")
+            logger.warning("Ignoring unit ex")
             text = text.replace("ex", '')
 
         text = text.strip()
@@ -375,7 +394,7 @@ class Svg2RlgAttributeConverter(AttributeConverter):
             tup = (float(val)/100.0 for val in t.split(','))
             return self.color_converter(colors.Color(*tup))
 
-        logger.warn("Can't handle color: %s" % text)
+        logger.warning("Can't handle color: %s" % text)
 
         return None
 
@@ -417,7 +436,7 @@ class Svg2RlgAttributeConverter(AttributeConverter):
         if non_exact_matches:
             return non_exact_matches[0]
         else:
-            logger.warn("Unable to find a suitable font for 'font-family:%s'" % svgAttr)
+            logger.warning("Unable to find a suitable font for 'font-family:%s'" % svgAttr)
             return DEFAULT_FONT_NAME
 
 
@@ -442,7 +461,7 @@ class ElementWrapper(object):
     @property
     def parent(self):
         par = self.object.getparent()
-        return ElementWrapper(par) if par else None
+        return ElementWrapper(par) if par is not None else None
 
     @property
     def classes(self):
@@ -489,6 +508,9 @@ class NodeTracker(ElementWrapper):
         super(NodeTracker, self).__init__(obj)
         self.usedAttrs = []
 
+    def __repr__(self):
+        return '<NodeTracker for node %s>' % self.object
+
     def getAttribute(self, name):
         # add argument to the history, if not already present
         if name not in self.usedAttrs:
@@ -501,6 +523,23 @@ class NodeTracker(ElementWrapper):
         return getattr(self.object, name)
 
 
+class CircularRefError(Exception):
+    pass
+
+
+class ExternalSVG:
+    def __init__(self, path, renderer):
+        self.root_node = load_svg_file(path)
+        self.renderer = SvgRenderer(path, parent_svgs=renderer._parent_chain + [renderer.source_path])
+        self.rendered = False
+
+    def get_fragment(self, fragment):
+        if not self.rendered:
+            self.renderer.render(self.root_node)
+            self.rendered = True
+        return self.renderer.definitions.get(fragment)
+
+
 ### the main meat ###
 
 class SvgRenderer:
@@ -510,23 +549,30 @@ class SvgRenderer:
     transforming it into a ReportLab Drawing instance.
     """
 
-    def __init__(self, path=None,color_converter=None):
+    def __init__(self, path, color_converter=None, parent_svgs=None):
+        self.source_path = path
+        self._parent_chain = parent_svgs or []  #  To detect circular refs.
         self.attrConverter = Svg2RlgAttributeConverter(color_converter=color_converter)
-        self.shape_converter = Svg2RlgShapeConverter(path,self.attrConverter)
+        self.shape_converter = Svg2RlgShapeConverter(path, self.attrConverter)
         self.handled_shapes = self.shape_converter.get_handled_shapes()
         self.definitions = {}
         self.waiting_use_nodes = defaultdict(list)
+        self._external_svgs = {}
 
     def render(self, svg_node):
         node = NodeTracker(svg_node)
+        view_box = self.get_box(node, default_box=True)
+        # Knowing the main box is useful for percentage units
+        self.attrConverter.set_box(view_box)
+
         main_group = self.renderSvg(node, outermost=True)
         for xlink in self.waiting_use_nodes.keys():
             logger.debug("Ignoring unavailable object width ID '%s'." % xlink)
 
-        view_box = self.get_box(node, default_box=True)
-        main_group.scale(1, -1)
         main_group.translate(0 - view_box.x, -view_box.height - view_box.y)
-        drawing = Drawing(view_box.width, view_box.height)
+
+        width, height = self.shape_converter.convert_length_attrs(svg_node, "width", "height")
+        drawing = Drawing(width, height)
         drawing.add(main_group)
         return drawing
 
@@ -539,7 +585,8 @@ class SvgRenderer:
 
         clipping = self.get_clippath(n)
         if name == "svg":
-            return self.renderSvg(n)
+            item = self.renderSvg(n)
+            parent.add(item)
         elif name == "defs":
             item = self.renderG(n)
         elif name == 'a':
@@ -561,8 +608,26 @@ class SvgRenderer:
         elif name == "clipPath":
             item = self.renderG(n)
         elif name in self.handled_shapes:
-            display = n.getAttribute("display")
+            if name == 'image':
+                # We resolve the image target at renderer level because it can point
+                # to another SVG file or node which has to be rendered too.
+                target = self.xlink_href_target(n)
+                if target is None:
+                    return
+                elif isinstance(target, tuple):
+                    # This is SVG content needed to be rendered
+                    gr = Group()
+                    renderer, node = target
+                    renderer.renderNode(node, parent=gr)
+                    self.apply_node_attr_to_group(n, gr)
+                    parent.add(gr)
+                    return
+                else:
+                    # Attaching target to node, so we can get it back in convertImage
+                    n._resolved_target = target
+
             item = self.shape_converter.convertShape(name, n, clipping)
+            display = n.getAttribute("display")
             if item and display != "none":
                 parent.add(item)
         else:
@@ -583,24 +648,54 @@ class SvgRenderer:
         Return the clipping Path object referenced by the node 'clip-path'
         attribute, if any.
         """
-        def get_path_from_node(node):
+        def get_shape_from_group(group):
+            for elem in group.contents:
+                if isinstance(elem, Group):
+                    return get_shape_from_group(elem)
+                elif isinstance(elem, SolidShape):
+                    return elem
+
+        def get_shape_from_node(node):
             for child in node.getchildren():
                 if node_name(child) == 'path':
                     group = self.shape_converter.convertShape('path', NodeTracker(child))
                     return group.contents[-1]
+                elif node_name(child) == 'use':
+                    grp = self.renderUse(NodeTracker(child))
+                    return get_shape_from_group(grp)
+                elif node_name(child) == 'rect':
+                    return self.shape_converter.convertRect(NodeTracker(child))
                 else:
-                    return get_path_from_node(child)
+                    return get_shape_from_node(child)
 
         clip_path = node.getAttribute('clip-path')
-        if clip_path:
-            m = re.match(r'url\(#([^\)]*)\)', clip_path)
-            if m:
-                ref = m.groups()[0]
-                if ref in self.definitions:
-                    path = get_path_from_node(self.definitions[ref])
-                    if path:
-                        path = ClippingPath(copy_from=path)
-                        return path
+        if not clip_path:
+            return
+        m = re.match(r'url\(#([^\)]*)\)', clip_path)
+        if not m:
+            return
+        ref = m.groups()[0]
+        if not ref in self.definitions:
+            return
+
+        shape = get_shape_from_node(self.definitions[ref])
+        if isinstance(shape, Rect):
+            # It is possible to use a rect as a clipping path in an svg, so we
+            # need to convert it to a path for rlg.
+            x1, y1, x2, y2 = shape.getBounds()
+            cp = ClippingPath()
+            cp.moveTo(x1, y1)
+            cp.lineTo(x2, y1)
+            cp.lineTo(x2, y2)
+            cp.lineTo(x1, y2)
+            cp.closePath()
+            # Copy the styles from the rect to the clipping path.
+            copy_shape_properties(shape, cp)
+            return cp
+        elif isinstance(shape, Path):
+            return ClippingPath(copy_from=shape)
+        elif shape:
+            logging.error("Unsupported shape type %s for clipping" % shape.__class__.__name__)
 
     def print_unused_attributes(self, node, n):
         if logger.level > logging.DEBUG:
@@ -610,6 +705,94 @@ class SvgRenderer:
         if unused_attrs:
             logger.debug("Unused attrs: %s %s" % (node_name(n), unused_attrs))
 
+    def apply_node_attr_to_group(self, node, group):
+        getAttr = node.getAttribute
+        transform, x, y = map(getAttr, ("transform", "x", "y"))
+        if x or y:
+            transform += " translate(%s, %s)" % (x or '0', y or '0')
+        if transform:
+            self.shape_converter.applyTransformOnGroup(transform, group)
+
+    def xlink_href_target(self, node, group=None):
+        """
+        Return either:
+            - a tuple (renderer, node) when the the xlink:href attribute targets
+              a vector file or node
+            - the path to an image file for any raster image targets
+            - None if any problem occurs
+        """
+        xlink_href = node.attrib.get('{http://www.w3.org/1999/xlink}href')
+        if not xlink_href:
+            return None
+
+        # First handle any raster embedded image data
+        match = re.match(r"^data:image/(jpeg|png);base64", xlink_href)
+        if match:
+            img_format = match.groups()[0]
+            image_data = base64.decodestring(xlink_href[(match.span(0)[1] + 1):].encode('ascii'))
+            file_indicator, path = tempfile.mkstemp(suffix='.%s' % img_format)
+            with open(path, 'wb') as fh:
+                fh.write(image_data)
+            # Close temporary file (as opened by tempfile.mkstemp)
+            os.close(file_indicator)
+            # this needs to be removed later, not here...
+            # if exists(path): os.remove(path)
+            return path
+
+        # From here, we can assume this is a path.
+        if '#' in xlink_href:
+            iri, fragment = xlink_href.split('#', 1)
+        else:
+            iri, fragment = xlink_href, None
+
+        if iri:
+            # Only local relative paths are supported yet
+            if not isinstance(self.source_path, str):
+                logger.error(
+                    "Unable to resolve image path '%s' as the SVG source is not a file system path." % iri
+                )
+                return None
+            path = os.path.normpath(os.path.join(os.path.dirname(self.source_path), iri))
+            if not os.access(path, os.R_OK):
+                return None
+            if path == self.source_path:
+                # Self-referencing, ignore the IRI part
+                iri = None
+
+        if iri:
+            if path.endswith('.svg'):
+                if path in self._parent_chain:
+                    logger.error("Circular reference detected in file.")
+                    raise CircularRefError()
+                if path not in self._external_svgs:
+                    self._external_svgs[path] = ExternalSVG(path, self)
+                ext_svg = self._external_svgs[path]
+                if ext_svg.root_node is not None:
+                    if fragment:
+                        ext_frag = ext_svg.get_fragment(fragment)
+                        if ext_frag is not None:
+                            return ext_svg.renderer, ext_frag
+                    else:
+                        return ext_svg.renderer, ext_svg.root_node
+            else:
+                # A raster image path
+                try:
+                    # This will catch invalid images
+                    PDFImage(path, 0, 0)
+                except IOError:
+                    logger.error("Unable to read the image %s. Skipping..." % path)
+                    return None
+                return path
+
+        elif fragment:
+            # A pointer to an internal definition
+            if fragment in self.definitions:
+                return self, self.definitions[fragment]
+            else:
+                # The missing definition should appear later in the file
+                self.waiting_use_nodes[fragment].append((node, group))
+                return DELAYED
+
     def renderTitle_(self, node):
         # Main SVG title attr. could be used in the PDF document info field.
         pass
@@ -629,32 +812,34 @@ class SvgRenderer:
             return Box(0, 0, width, height)
 
     def renderSvg(self, node, outermost=False):
-        getAttr = node.getAttribute
-
         _saved_preserve_space = self.shape_converter.preserve_space
-        self.shape_converter.preserve_space = getAttr("{%s}space" % XML_NS) == 'preserve'
+        self.shape_converter.preserve_space = node.getAttribute("{%s}space" % XML_NS) == 'preserve'
 
         group = Group()
         for child in node.getchildren():
             self.renderNode(child, group)
         self.shape_converter.preserve_space = _saved_preserve_space
 
+        # Translating
         if not outermost:
-            x, y = map(getAttr, ("x", "y"))
-            x, y = map(self.attrConverter.convertLength, (x, y))
+            x, y = self.shape_converter.convert_length_attrs(node, "x", "y")
             if x or y:
                 group.translate(x or 0, y or 0)
 
-            view_box = self.get_box(node)
-            if view_box:
-                x_scale, y_scale = 1, 1
-                width, height = map(getAttr, ("width", "height"))
-                width, height = map(self.attrConverter.convertLength, (width, height))
-                if view_box.height != height:
-                    y_scale = height / view_box.height
-                if view_box.width != width:
-                    x_scale = width / view_box.width
-                group.scale(x_scale, y_scale)
+        # Scaling
+        view_box = self.get_box(node)
+        if not view_box and outermost:
+            # Apply only the 'reverse' y-scaling (PDF 0,0 is bottom left)
+            group.scale(1, -1)
+        elif view_box:
+            x_scale, y_scale = 1, 1
+            width, height = self.shape_converter.convert_length_attrs(node, "width", "height")
+            if view_box.height != height:
+                y_scale = height / view_box.height
+            if view_box.width != width:
+                x_scale = width / view_box.width
+            group.scale(x_scale, y_scale * (-1 if outermost else 1))
+
         return group
 
     def renderG(self, node, clipping=None, display=1):
@@ -688,34 +873,35 @@ class SvgRenderer:
         if group is None:
             group = Group()
 
-        xlink_href = node.attrib.get('{http://www.w3.org/1999/xlink}href')
-        if not xlink_href:
-            return
-        if xlink_href[1:] not in self.definitions:
-            # The missing definition should appear later in the file
-            self.waiting_use_nodes[xlink_href[1:]].append((node, group))
+        try:
+            item = self.xlink_href_target(node, group=group)
+        except CircularRefError:
+            node.parent.object.remove(node.object)
             return group
+        if item is None:
+            return
+        elif isinstance(item, str):
+            logger.error("<use> nodes cannot reference bitmap image files")
+            return
+        elif item is DELAYED:
+            return group
+        else:
+            item = item[1]  # [0] is the renderer, not used here.
 
         if clipping:
             group.add(clipping)
         if len(node.getchildren()) == 0:
             # Append a copy of the referenced node as the <use> child (if not already done)
-            node.append(copy.deepcopy(self.definitions[xlink_href[1:]]))
+            node.append(copy.deepcopy(item))
         self.renderNode(node.getchildren()[-1], parent=group)
-        getAttr = node.getAttribute
-        transform = getAttr("transform")
-        x, y = map(getAttr, ("x", "y"))
-        if x or y:
-            transform += " translate(%s, %s)" % (x or '0', y or '0')
-        if transform:
-            self.shape_converter.applyTransformOnGroup(transform, group)
+        self.apply_node_attr_to_group(node, group)
         return group
 
 
 class SvgShapeConverter:
     """An abstract SVG shape converter.
 
-    Implement subclasses with methods named 'renderX(node)', where
+    Implement subclasses with methods named 'convertX(node)', where
     'X' should be the capitalised name of an SVG node element for
     shapes, like 'Rect', 'Circle', 'Line', etc.
 
@@ -758,45 +944,37 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
             group.add(shape)
             return group
 
-    def convertLine(self, node):
-        getAttr = node.getAttribute
-        x1, y1, x2, y2 = map(getAttr, ("x1", "y1", "x2", "y2"))
-        x1, y1, x2, y2 = map(self.attrConverter.convertLength, (x1, y1, x2, y2))
-        shape = Line(x1, y1, x2, y2)
+    def convert_length_attrs(self, node, *attrs, **kwargs):
+        # When dropping Python 2, we can replace **kwargs by em_base=None in the
+        # function signature.
+        em_base = kwargs.pop('em_base', None)
+        # Support node both as NodeTracker or lxml node
+        getAttr = node.getAttribute if hasattr(node, 'getAttribute') else lambda attr: node.attrib.get(attr, '')
+        convLength = self.attrConverter.convertLength
+        return [convLength(getAttr(attr), attr_name=attr, em_base=em_base) for attr in attrs]
 
-        return shape
+    def convertLine(self, node):
+        x1, y1, x2, y2 = self.convert_length_attrs(node, 'x1', 'y1', 'x2', 'y2')
+        return Line(x1, y1, x2, y2)
 
     def convertRect(self, node):
-        getAttr = node.getAttribute
-        x, y, width, height = map(getAttr, ('x', 'y', "width", "height"))
-        x, y, width, height = map(self.attrConverter.convertLength, (x, y, width, height))
-        rx, ry = map(getAttr, ("rx", "ry"))
-        rx, ry = map(self.attrConverter.convertLength, (rx, ry))
-        shape = Rect(x, y, width, height, rx=rx, ry=ry)
-
-        return shape
+        x, y, width, height, rx, ry = self.convert_length_attrs(
+            node, 'x', 'y', 'width', 'height', 'rx', 'ry'
+        )
+        return Rect(x, y, width, height, rx=rx, ry=ry)
 
     def convertCircle(self, node):
         # not rendered if r == 0, error if r < 0.
-        getAttr = node.getAttribute
-        cx, cy, r = map(getAttr, ("cx", "cy", 'r'))
-        cx, cy, r = map(self.attrConverter.convertLength, (cx, cy, r))
-        shape = Circle(cx, cy, r)
-
-        return shape
+        cx, cy, r = self.convert_length_attrs(node, 'cx', 'cy', 'r')
+        return Circle(cx, cy, r)
 
     def convertEllipse(self, node):
-        getAttr = node.getAttribute
-        cx, cy, rx, ry = map(getAttr, ("cx", "cy", "rx", "ry"))
-        cx, cy, rx, ry = map(self.attrConverter.convertLength, (cx, cy, rx, ry))
+        cx, cy, rx, ry = self.convert_length_attrs(node, 'cx', 'cy', 'rx', 'ry')
         width, height = rx, ry
-        shape = Ellipse(cx, cy, width, height)
-
-        return shape
+        return Ellipse(cx, cy, width, height)
 
     def convertPolyline(self, node):
-        getAttr = node.getAttribute
-        points = getAttr("points")
+        points = node.getAttribute("points")
         points = points.replace(',', ' ')
         points = points.split()
         points = list(map(self.attrConverter.convertLength, points))
@@ -822,8 +1000,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
         return polyline
 
     def convertPolygon(self, node):
-        getAttr = node.getAttribute
-        points = getAttr("points")
+        points = node.getAttribute("points")
         points = points.replace(',', ' ')
         points = points.split()
         points = list(map(self.attrConverter.convertLength, points))
@@ -866,9 +1043,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
         ff = attrConv.convertFontFamily(ff)
         fs = attrConv.findAttr(node, "font-size") or "12"
         fs = attrConv.convertLength(fs)
-        convertLength = partial(attrConv.convertLength, em_base=fs)
-        x, y = map(node.getAttribute, ('x', 'y'))
-        x, y = map(convertLength, (x, y))
+        x, y = self.convert_length_attrs(node, 'x', 'y', em_base=fs)
         for c in itertools.chain([node], node.getchildren()):
             has_x, has_y = False, False
             dx, dy = 0, 0
@@ -881,16 +1056,15 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
                 text = self.clean_text(c.text, preserve_space)
                 if not text:
                     continue
-                x1, y1, dx, dy = [c.attrib.get(name, '') for name in ("x", "y", "dx", "dy")]
-                has_x, has_y = (x1 != '', y1 != '')
-                x1, y1, dx, dy = map(convertLength, (x1, y1, dx, dy))
+                x1, y1, dx, dy = self.convert_length_attrs(c, 'x', 'y', 'dx', 'dy', em_base=fs)
+                has_x, has_y = (c.attrib.get('x', '') != '', c.attrib.get('y', '') != '')
                 dx0 = dx0 + dx
                 dy0 = dy0 + dy
                 baseLineShift = c.attrib.get("baseline-shift", '0')
                 if baseLineShift in ("sub", "super", "baseline"):
                     baseLineShift = {"sub":-fs/2, "super":fs/2, "baseline":0}[baseLineShift]
                 else:
-                    baseLineShift = convertLength(baseLineShift, fs)
+                    baseLineShift = attrConv.convertLength(baseLineShift, em_base=fs)
             else:
                 continue
 
@@ -1071,38 +1245,17 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
 
     def convertImage(self, node):
         if not haveImages:
-            logger.warn(
+            logger.warning(
                 "Unable to handle embedded images. Maybe the pillow library is missing?"
             )
             return None
 
-        getAttr = node.getAttribute
-        x, y, width, height = map(getAttr, ('x', 'y', "width", "height"))
-        x, y, width, height = map(self.attrConverter.convertLength, (x, y, width, height))
-        xlink_href = node.attrib.get('{http://www.w3.org/1999/xlink}href')
+        x, y, width, height = self.convert_length_attrs(node, 'x', 'y', 'width', 'height')
+        image = node._resolved_target
+        image = Image(int(x), int(y + height), int(width), int(height), image)
 
-        magic_re = r"^data:image/(jpeg|png);base64"
-        match = re.match(magic_re, xlink_href)
-        if match:
-            img_format = match.groups()[0]
-            image_data = base64.decodestring(xlink_href[(match.span(0)[1] + 1):].encode('ascii'))
-            _, path = tempfile.mkstemp(suffix='.%s' % img_format)
-            with open(path, 'wb') as fh:
-                fh.write(image_data)
-            img = Image(int(x), int(y + height), int(width), int(height), path)
-            # this needs to be removed later, not here...
-            # if exists(path): os.remove(path)
-        else:
-            xlink_href = os.path.join(os.path.dirname(self.svg_source_file), xlink_href)
-            img = Image(int(x), int(y + height), int(width), int(height), xlink_href)
-            try:
-                # this will catch invalid image
-                PDFImage(xlink_href, 0, 0)
-            except IOError:
-                logger.error("Unable to read the image %s. Skipping..." % img.path)
-                return None
-        group = Group(img)
-        group.translate(0, (x + height) * 2)
+        group = Group(image)
+        group.translate(0, (y + height) * 2)
         group.scale(1, -1)
         return group
 
@@ -1210,18 +1363,13 @@ def svg2rlg(path, **kwargs):
         path = path[:-1]
         unzipped = True
 
-    # load SVG file
-    parser = etree.XMLParser(remove_comments=True, recover=True)
-    try:
-        doc = etree.parse(path, parser=parser)
-        svg = doc.getroot()
-    except Exception as exc:
-        logger.error("Failed to load input file! (%s)" % exc)
+    svg_root = load_svg_file(path)
+    if svg_root is None:
         return
 
     # convert to a RLG drawing
-    svgRenderer = SvgRenderer(path,**kwargs)
-    drawing = svgRenderer.render(svg)
+    svgRenderer = SvgRenderer(path, **kwargs)
+    drawing = svgRenderer.render(svg_root)
 
     # remove unzipped .svgz file (.svg)
     if unzipped:
@@ -1230,6 +1378,17 @@ def svg2rlg(path, **kwargs):
     return drawing
 
 
+def load_svg_file(path):
+    parser = etree.XMLParser(remove_comments=True, recover=True)
+    try:
+        doc = etree.parse(path, parser=parser)
+        svg_root = doc.getroot()
+    except Exception as exc:
+        logger.error("Failed to load input file! (%s)" % exc)
+    else:
+        return svg_root
+
+
 def node_name(node):
     """Return lxml node name without the namespace prefix."""
 
@@ -1239,6 +1398,14 @@ def node_name(node):
         pass
 
 
+def copy_shape_properties(source_shape, dest_shape):
+    for prop, val in source_shape.getProperties().items():
+        try:
+            setattr(dest_shape, prop, val)
+        except AttributeError:
+            pass
+
+
 def monkeypatch_reportlab():
     """
     https://bitbucket.org/rptlab/reportlab/issues/95/
