@@ -16,7 +16,7 @@ def split_floats(op, min_num, value):
     Example: with op='m' and value='10,20 30,40,' the returned value will be
              ['m', [10.0, 20.0], 'l', [30.0, 40.0]]
     """
-    floats = [float(seq) for seq in re.findall('(-?\d*\.?\d*(?:e[+-]\d+)?)', value) if seq]
+    floats = [float(seq) for seq in re.findall(r'(-?\d*\.?\d*(?:e[+-]\d+)?)', value) if seq]
     res = []
     for i in range(0, len(floats), min_num):
         if i > 0 and op in {'m', 'M'}:
@@ -147,6 +147,7 @@ def end_point_to_center_parameters(x1, y1, x2, y2, fA, fS, rx, ry, phi=0):
         rx *= rr
         ry *= rr
         r = x1d * x1d / (rx * rx) + y1d * y1d / (ry * ry)
+        r = 1 / r - 1
     elif r != 0:
         r = 1 / r - 1
     if -1e-10 < r < 0:
