@@ -162,6 +162,30 @@ class TestPaths(object):
         group = converter.convertPath(node)
         assert group is None
 
+    def test_clipped_path(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+            <?xml version="1.0"?>
+            <svg version="1.1" xmlns="http://www.w3.org/2000/svg"
+                 xmlns:xlink="http://www.w3.org/1999/xlink" width="660" height="480">
+                <defs>
+                    <path id="my-clipping-path" d="M99,176 L 110 170 112 172Z"/>
+                    <rect id="my-clipping-rect" x="88.155" y="163" width="419.69" height="20.68"/>
+                </defs>
+                <clipPath id="my-clip-path">
+                    <use xlink:href="#my-clipping-path"/>
+                </clipPath>
+                <clipPath id="my-clip-rect">
+                    <use xlink:href="#my-clipping-rect"/>
+                </clipPath>
+                <path clip-path="url(#my-clip-path)" d="M99,176 L 110 170 112 172Z"/>
+                <path clip-path="url(#my-clip-rect)" d="M99,176 L 110 170 112 172Z"/>
+            </svg>
+        ''')))
+        assert isinstance(drawing.contents[0].contents[0].contents[0], svglib.ClippingPath)
+        # Clipping rect was converted to a path
+        assert isinstance(drawing.contents[0].contents[1].contents[0], svglib.ClippingPath)
+
+
 def force_cmyk(rgb):
     c, m, y, k = colors.rgb2cmyk(rgb.red,rgb.green,rgb.blue)
     return colors.CMYKColor(c,m,y,k,alpha=rgb.alpha)
@@ -215,7 +239,6 @@ class TestLengthAttrConverter(object):
             ("1e-5", 1e-5),
             ("1e1cm", 10*cm),
             ("1e1in", 10*inch),
-            ("1e1%", 10),
             ("-8e-2cm", (-8e-2)*cm),
             ("20px", 20),
             ("20pt", 20 * 1.25),
@@ -227,14 +250,16 @@ class TestLengthAttrConverter(object):
         assert len(failed) == 0
         assert ac.convertLength("1.5em", em_base=16.5) == 24.75
 
-    def test_1(self):
-        "Test length attribute conversion."
+    def test_percentage_conversion(self):
+        "Test percentage length attribute conversion."
 
         ac = svglib.Svg2RlgAttributeConverter()
-        attr = "1e1%"
-        expected = 1
-        obj = ac.convertLength(attr, 10)
-        assert obj == expected
+        ac.set_box(svglib.Box(0, 0, 50, 150))
+        # Percentages depend on current viewport and type of attribute
+        length = ac.convertLength("1e1%", attr_name='width')
+        assert length == 5
+        length = ac.convertLength("1e1%", attr_name='height')
+        assert length == 15
 
 
 class TestLengthListAttrConverter(object):
@@ -649,10 +674,23 @@ class TestViewBox(object):
             </svg>
         ''')))
         # Main group coordinates are translated to match the viewBox origin
-        assert drawing.contents[0].transform == (1, 0, 0, -1, 60.0, 40.0)
+        assert drawing.contents[0].transform == (10, 0, 0, -10, 600.0, 400.0)
+
+    def test_percent_width_height(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+            <?xml version="1.0"?>
+            <svg xmlns="http://www.w3.org/2000/svg"
+                 width="100%" height="100%"
+                 viewBox="0 0 480 360">
+                <g fill="#E70013">
+                    <rect x="60" y="40" width="120" height="80"/>
+                </g>
+            </svg>
+        ''')))
+        assert (drawing.width, drawing.height) == (480, 360)
 
 
-class TestSVGEmbedded(object):
+class TestEmbedded(object):
     def test_svg_in_svg(self):
         drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
             <?xml version="1.0"?>
@@ -684,3 +722,16 @@ class TestSVGEmbedded(object):
         # viewBox scaling
         assert pytest.approx(1.417, 0.001) == embedded_svg_group.getProperties()['transform'][0]
         assert pytest.approx(1.417, 0.001) == embedded_svg_group.getProperties()['transform'][3]
+
+    def test_png_in_svg_file_like(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent(u'''\
+            <?xml version="1.0"?>
+            <svg xmlns="http://www.w3.org/2000/svg" version="1.1"
+                 xmlns:xlink="http://www.w3.org/1999/xlink"
+                 viewBox="0 0 210 297" height="297mm" width="210mm">
+              <image id="refImage" xlink:href="../png/jpeg-required-201-t.png" height="36" width="48" y="77" x="50" />
+            </svg>
+        ''')))
+        # FIXME: test the error log when we can require pytest >= 3.4
+        # No image as relative path in file-like input cannot be determined.
+        assert drawing.contents[0].contents == []
